@@ -4,20 +4,17 @@ import sys
 import traceback
 from logging import getLogger
 
+from django.conf.urls.defaults import url
 from django.core.exceptions import ObjectDoesNotExist, MultipleObjectsReturned, ValidationError
 from django.core.mail import mail_admins
-from django.forms.models import ModelChoiceField
 from django.http import HttpResponse
-from django.utils.encoding import smart_unicode
 from django.views.decorators.csrf import csrf_exempt
 from django.views.generic import View
 
-from .utils import status, MetaOptions
-from .utils.auth import AnonimousAuthenticator
-from .utils.emitter import HTMLTemplateEmitter, JSONEmitter
+from .utils import status, MetaOptions, gen_url_name, gen_url_regex
 from .utils.exceptions import HttpError
-from .utils.tools import as_tuple
 from .utils.paginator import Paginator
+from .utils.tools import as_tuple
 from adrest import settings
 from adrest.mixin import auth, emitter, handler, parser, throttle
 from adrest.signals import api_request_started, api_request_finished
@@ -33,39 +30,47 @@ class ResourceMetaClass(handler.HandlerMeta, throttle.ThrottleMeta, emitter.Emit
     """
 
     def __new__(mcs, name, bases, params):
+
+        # Create meta if not exists
         params['meta'] = params.get('meta', MetaOptions())
 
+        # Run other meta classes
         cls = super(ResourceMetaClass, mcs).__new__(mcs, name, bases, params)
-        cls.allowed_methods = as_tuple(cls.allowed_methods)
-        if settings.ALLOW_OPTIONS:
-            cls.allowed_methods += 'OPTIONS',
 
+        # Prepare allowed methods
+        cls.allowed_methods = mcs.prepare_methods(cls.allowed_methods)
+
+        # Check parent
         if cls.parent:
             try:
                 cls.meta.parents += cls.parent.meta.parents + [cls.parent]
             except AttributeError:
                 raise TypeError("%s.parent must be instance of %s" % (name, "ResourceView"))
 
+        # Meta name (maybe precalculate in handler)
         cls.meta.name = cls.meta.name or cls.name or ''.join(bit for bit in name.split('Resource') if bit).lower()
-        uri_params = cls.uri_params or []
-        cls.meta.urlname = '-'.join(
-                ([cls.parent.meta.urlname] if cls.parent else []) +
-                ([ cls.prefix ] if cls.prefix else []) +
-                list(uri_params) +
-                [cls.meta.name])
-        cls.meta.urlregex = '/'.join(
-                '%(name)s/(?P<%(name)s>[^/]+)' % dict(name = p)
-                for p in (
-                    [p.meta.name for p in cls.meta.parents] + list(uri_params)))
-        cls.meta.urlregex = cls.meta.urlregex + '/' if cls.meta.urlregex else ''
-        if cls.prefix:
-            cls.meta.urlregex = '%s%s/' % (cls.meta.urlregex, cls.prefix)
-        cls.meta.urlregex += '%(name)s/(?:(?P<%(name)s>[^/]+)/)?$' % dict(
-                name = cls.meta.name)
 
-        cls.meta.models = [o.model for o in cls.meta.parents + [ cls ] if o.model]
+        # Prepare urls
+        cls.url_params = list(as_tuple(cls.url_params))
+        cls.meta.url_name = cls.url_name or '-'.join(gen_url_name(cls))
+        cls.meta.url_regex = cls.url_regex or '/'.join(gen_url_regex(cls))
+
         return cls
 
+    @staticmethod
+    def prepare_methods(methods):
+        " Prepare allowed methods. "
+
+        methods = as_tuple(methods)
+
+        if settings.ALLOW_OPTIONS:
+            methods += 'OPTIONS',
+
+        if 'GET' in methods:
+            methods += 'HEAD',
+
+        return methods
+
 
 class ResourceView(handler.HandlerMixin,
         throttle.ThrottleMixin,
@@ -96,14 +101,16 @@ class ResourceView(handler.HandlerMixin,
     prefix = ''
 
     # Some custom URI params here
-    uri_params = None
+    url_params = None
+    url_regex = None
+    url_name = None
 
     # If children object in hierarchy has FK=Null to parent, allow to get this
     # object (default: True)
     allow_public_access = False
 
     @csrf_exempt
-    def dispatch(self, request, *args, **kwargs):
+    def dispatch(self, request, **resources):
 
         # Send started signal
         api_request_started.send(self, request = request)
@@ -127,7 +134,7 @@ class ResourceView(handler.HandlerMixin,
             self.throttle_check()
 
             # Get required resources
-            resources = self.get_resources_from_uri(**kwargs)
+            resources = self.get_resources(request, **resources)
 
             # Check owners
             self.check_owners(**resources)
@@ -145,7 +152,7 @@ class ResourceView(handler.HandlerMixin,
             func = getattr(self, self.callmap.get(method, None))
 
             # Get function data
-            content = func(request, *args, **resources)
+            content = func(request, **resources)
 
             # Serialize response
             response = self.emit(content, request=request)
@@ -180,51 +187,41 @@ class ResourceView(handler.HandlerMixin,
 
         return response
 
-    def check_method_allowed(self, method):
+    @classmethod
+    def check_method_allowed(cls, method):
         """ Ensure the request HTTP method is permitted for this resource, raising a ResourceException if it is not.
         """
-        if not method in self.callmap.keys():
+        if not method in cls.callmap.keys():
             raise HttpError('Unknown or unsupported method \'%s\'' % method, status=status.HTTP_501_NOT_IMPLEMENTED)
 
-        if not method in self.allowed_methods:
+        if not method in cls.allowed_methods:
             raise HttpError('Method \'%s\' not allowed on this resource.' % method, status=status.HTTP_405_METHOD_NOT_ALLOWED)
 
-    def get_resources_from_uri(self, **resources):
-        """ Parse form params from URI.
+    @classmethod
+    def get_resources(cls, request, **resources):
+        " Parse resource objects from URL and GET. "
 
-            For example, /author/1/book/2 converted to resources array with ORM
-            objects Book with pk=2 and Author with pk=1
+        # Get from parent
+        if cls.parent:
+            resources = cls.parent.get_resources(request, **resources)
 
-        """
+        pk = resources.get(cls.meta.name) or request.GET.get(cls.meta.name)
+        if not cls.model or not pk:
+            return resources
 
-        # Generate pairs (model_name, model_class) and convert it to dictionary
-        mdict = dict((m._meta.module_name, m) for m in self.meta.models)
+        try:
+            resources[cls.meta.name] = cls.model.objects.get(pk = pk)
 
-        # URL objects
-        for key, value in resources.iteritems():
-            key = key.lower()
-            # If recource view associated with this model and we have its value
-            # in URI -- try to convert it in real ORM object
-            if mdict.has_key(key) and value:
-                try:
-                    resources[key] = mdict.get(key).objects.get(pk=value)
+        except (ObjectDoesNotExist, ValueError):
+            raise HttpError("Resource not found.", status=status.HTTP_404_NOT_FOUND)
 
-                except (ObjectDoesNotExist, ValueError):
-                    raise HttpError("Resource not found.", status=status.HTTP_404_NOT_FOUND)
+        except MultipleObjectsReturned:
+            raise HttpError("Resources conflict.", status=status.HTTP_409_CONFLICT)
 
-                except MultipleObjectsReturned:
-                    raise HttpError("Resources conflict.", status=status.HTTP_409_CONFLICT)
-
-        # If we get ORM object resource with model_name like this resource model
-        # name, add it to resource as "instance"
-        # For example: ResourceView.model=Author, request was /author/1/
-        # resources['instance'] = Author_object_with_pk=1
-        instance = resources.get(self.meta.name)
-        if instance:
-            resources['instance'] = instance
         return resources
 
-    def check_owners(self, **resources):
+    @classmethod
+    def check_owners(cls, **resources):
         """ Recursive scanning of the fact that the child has FK
             to the parent and in resources we have right objects.
 
@@ -234,37 +231,20 @@ class ResourceView(handler.HandlerMixin,
             and Book with pk=2 has ForeignKey field linked to Author object with pk=1.
         """
 
-        # Build iterator with all models for this Resource
-        # Models list generated automatically based on "parent" and "model field
-        # in Resource description
-        it = reversed( [m._meta.module_name for m in self.meta.models] )
-        try:
-            # Get two models names
-            c_name, f_name = next(it), next(it)
+        resource = resources.get(cls.meta.name)
+        if cls.allow_public_access or not cls.parent or not resource:
+            return True
 
-            while True:
-                # Get objects from resources array for two models
-                ofm, ocm = resources.get(f_name), resources.get(c_name)
+        cls.parent.check_owners(**resources)
 
-                # Test parent element linked from children
-                # If children haven't link to parent and it's allowed by allow_public_access -- it's ok, object available for public
-                # If children have FK field named as parent model and it's value equivalent to
-                #   object in resources -- it's OK
-                # ELSE -- it's not ok, stop check!
-                parent_in_child = getattr(ocm, '%s_id' % f_name, None)
-                assert (not ocm) or \
-                       (self.allow_public_access and self.allow_public_access == c_name and not parent_in_child) or \
-                       (parent_in_child and ofm and parent_in_child == ofm.pk)
-
-                # Swap and get one more model name from iterator
-                f_name, c_name = next(it), f_name
-
-        except (AssertionError, ObjectDoesNotExist):
-            # 403 Error if there is error in parent-children relationship
-            raise HttpError("Access forbidden.", status=status.HTTP_403_FORBIDDEN)
-
-        except StopIteration:
-            pass
+        if cls.model and cls.parent.model:
+            parent_resource = resources.get(cls.parent.meta.name)
+            parent_resource_id = getattr(resource, "%s_id" % cls.parent.meta.name, None)
+            try:
+                assert parent_resource and parent_resource.pk == parent_resource_id
+            except AssertionError:
+                # 403 Error if there is error in parent-children relationship
+                raise HttpError("Access forbidden.", status=status.HTTP_403_FORBIDDEN)
 
         return True
 
@@ -285,42 +265,16 @@ class ResourceView(handler.HandlerMixin,
     def version(self):
         return str(self.api or '')
 
+    @classmethod
+    def as_url(cls, api=None, name_prefix='', url_prefix=''):
+        " Generate url for resource. "
+        url_prefix = url_prefix and "%s/" % url_prefix
+        name_prefix = name_prefix and "%s-" % name_prefix
 
-class ApiMapResource(ResourceView):
-    """ Simple JSON Api Map.
-    """
-    log = False
-    emitters = HTMLTemplateEmitter, JSONEmitter
-    authenticators = AnonimousAuthenticator
-    template = 'api/apimap.html'
+        url_regex = '^%s%s$' % (url_prefix, cls.meta.url_regex.lstrip('^').rstrip('/$'))
+        url_name = '%s%s' % (name_prefix, cls.meta.url_name)
 
-    def get(self, *args, **Kwargs):
-        return self.api.str_version, list(self.gen_apimap())
-
-    def gen_apimap(self):
-        for urlname in sorted(self.api._map.iterkeys()):
-            urlregex, resource = self.api._map[urlname]
-            info = dict(
-                name = urlname,
-                methods = resource.allowed_methods,
-                fields = []
-            )
-            if resource.model:
-                info['resource'] = resource.model.__name__
-
-            if resource.form and ('POST' in resource.allowed_methods or 'PUT' in resource.allowed_methods):
-                info['fields'] += [
-                    (name, dict(required = f.required and f.initial is None, help = smart_unicode(f.help_text + '')))
-                        for name, f in resource.form.base_fields.iteritems()
-                        if not (isinstance(f, ModelChoiceField) and f.choices.queryset.model in resource.meta.models)
-                ]
-
-            for a in resource.authenticators:
-                info['fields'] += a.get_fields()
-
-            info['auth'] = set(map(str, resource.authenticators))
-            key = urlregex.replace("(?P", "").replace("[^/]+)", "").replace("?:", "").replace("$", "")
-            yield key, info
+        return url(url_regex, cls.as_view(api=api), name = url_name)
 
 
 def errors_mail(response, request):
